<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Filesystem\File;

// Import framework stuff
use Org\Mxchange\CoreFramework\Filesystem\Filesystem;

/**
 * A virtual file system interface
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2022 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
interface BinaryFile extends Filesystem {
	/**
	 * Separator for header data
	 */
	const SEPARATOR_HEADER_DATA = 0x01;

	/**
	 * Separator header->entries
	 */
	const SEPARATOR_HEADER_ENTRIES = 0x02;

	/**
	 * Separator group->hash
	 */
	const SEPARATOR_GROUP_HASH = 0x03;

	/**
	 * Separator hash->value
	 */
	const SEPARATOR_HASH_VALUE = 0x04;

	/**
	 * Separator entry->entry
	 */
	const SEPARATOR_ENTRIES = 0x05;

	/**
	 * Separator type->position
	 */
	const SEPARATOR_TYPE_POSITION = 0x06;

	/**
	 * Length of count
	 */
	const LENGTH_COUNT = 20;

	/**
	 * Length of position
	 */
	const LENGTH_POSITION = 20;

	/**
	 * Length of group
	 */
	const LENGTH_GROUP = 10;

	/**
	 * Maximum length of entry type
	 */
	const LENGTH_TYPE = 20;

	//***** Array elements for 'gaps' array *****

	/**
	 * Start of gap
	 */
	const GAPS_INDEX_START = 'start';

	/**
	 * End of gap
	 */
	const GAPS_INDEX_END = 'end';

	// Header names
	const HEADER_NAME_MAGIC = 'magic';
	const HEADER_NAME_TOTAL_ENTRIES = 'total';
	const HEADER_NAME_SEEK_POSITION = 'seek';

	// Header element counts
	const HEADER_INDEX_ELEMENT_COUNT = 2;
	const HEADER_STACK_ELEMENT_COUNT = 3;

	/**
	 * Reads the file header
	 *
	 * @return	void
	 */
	function readFileHeader ();

	/**
	 * Flushes the file header
	 *
	 * @return	void
	 */
	function flushFileHeader ();

	/**
	 * Determines whether the EOF has been reached
	 *
	 * @return	$isEndOfFileReached		Whether the EOF has been reached
	 */
	function isEndOfFileReached ();

	/**
	 * Getter for header size
	 *
	 * @return	$totalEntries	Size of file header
	 */
	function getHeaderSize ();

	/**
	 * Setter for header size
	 *
	 * @param	$headerSize		Size of file header
	 * @return	void
	 */
	function setHeaderSize (int $headerSize);

	/**
	 * Getter for header array
	 *
	 * @return	$totalEntries	Size of file header
	 */
	function getHeader ();

	/**
	 * Setter for header
	 *
	 * @param	$header		Array for a file header
	 * @return	void
	 */
	function setHeader (array $header);

	/**
	 * Initializes counter for valid entries, arrays for damaged entries and
	 * an array for gap seek positions. If you call this method on your own,
	 * please re-analyze the file structure. So you are better to call
	 * analyzeFileStructure() instead of this method.
	 *
	 * @return	void
	 */
	function initCountersGapsArray ();

	/**
	 * Updates seekPosition attribute from file to avoid to much access on file.
	 *
	 * @return	void
	 */
	function updateSeekPosition ();

	/**
	 * Getter for total entries
	 *
	 * @return	$totalEntries	Total entries in this file
	 */
	function getCounter ();

	/**
	 * "Getter" for file size
	 *
	 * @return	$fileSize	Size of currently loaded file
	 */
	function getFileSize ();

	/**
	 * Checks whether the abstracted file only contains gaps by counting all
	 * gaps' bytes together and compare it to total length.
	 *
	 * @return	$isGapsOnly		Whether the abstracted file only contains gaps
	 */
	function isFileGapsOnly();

	/**
	 * Searches for next suitable gap the given length of data can fit in
	 * including padding bytes.
	 *
	 * @param	$length			Length of raw data
	 * @return	$seekPosition	Found next gap's seek position
	 */
	function searchNextGap (int $length);

	/**
	 * Writes given value to the file and returns a hash and gap position for it
	 *
	 * @param	$groupId	Group identifier
	 * @param	$value		Value to be added to the stack
	 * @return	$data		Hash and gap position
	 */
	function writeValueToFile (string $groupId, string $rawData);

	/**
	 * Writes given raw data to the file and returns a gap position and length
	 *
	 * @param	$groupId	Group identifier
	 * @param	$hash		Hash from encoded value
	 * @param	$encoded	Encoded value to be written to the file
	 * @return	$data		Gap position and length of the raw data
	 */
	function writeDataToFreeGap (string $groupId, string $hash, string $encoded);

	/**
	 * Writes data at given position
	 *
	 * @param	$seekPosition	Seek position
	 * @param	$data			Data to be written
	 * @param	$flushHeader	Whether to flush the header (default: flush)
	 * @return	void
	 */
	function writeData (int $seekPosition, string $data, bool $flushHeader = true);

	/**
	 * Writes at given position by seeking to it.
	 *
	 * @param	$seekPosition	Seek position in file
	 * @param	$dataStream		Data to be written
	 * @return	mixed			Number of writes bytes or false on error
	 * @throws	InvalidArgumentException	If a parameter is not valid
	 */
	function writeAtPosition (int $seekPosition, string $dataStream);

}
